"""Base classes for models."""
from typing import Any

from sqlalchemy import Column, Integer, MetaData, Table, Text
from sqlalchemy.orm import as_declarative, declared_attr

NAMING_CONVENTION = {
    'pk': '%(table_name)s_pkey',
    'ix': '%(table_name)s_%(column_0_name)s_key',
    'fk': '%(table_name)s_%(column_0_name)s_fk',
    'ck': '%(table_name)s_%(constraint_name)s',
    'uq': '%(table_name)s_%(column_0_name)s_key',
}
METADATA = MetaData(naming_convention=NAMING_CONVENTION)


APPLICATION_PREFIX = 'messenger_'


@as_declarative(metadata=METADATA)
class BaseModel:
    """Base class for messenger models."""

    id = Column(Integer, primary_key=True)

    @declared_attr
    def __tablename__(cls) -> str:
        return APPLICATION_PREFIX + cls.__name__.lower()  # type: ignore

    def __init__(self, *args: Any, **kwargs: Any):
        # Define the __init__ to provide definition for mypy
        super().__init__(*args, **kwargs)  # pragma: no cover


# Map the SQLite internal table 'sqlite_sequence'.
SQLITE_SEQUENCE = Table(
    'sqlite_sequence', METADATA,
    Column('name', Text),
    Column('seq', Integer),
)
