"""Models for SMS messages."""
from sqlalchemy import Column, ForeignKey, ForeignKeyConstraint, Integer, Text
from sqlalchemy.orm import relationship

from .base import BaseModel
from .message import ArchiveReferenceMixin, ReferenceMixin, TemplateArchiveMessageMixin, TemplateMessageMixin


class SmsMixin:
    """Shared columns between SMS and archive SMS."""

    # Message attributes
    recipient = Column(Text(), nullable=False)


class Sms(SmsMixin, TemplateMessageMixin, BaseModel):
    """Represents an SMS message."""

    references = relationship('SmsReference')


class SmsReference(ReferenceMixin, BaseModel):
    """Represents a SMS message reference."""

    message_id = Column(ForeignKey(Sms.__table__.columns.id), nullable=False)  # type: ignore[attr-defined]


class ArchiveSms(SmsMixin, TemplateArchiveMessageMixin, BaseModel):
    """Represents an archived SMS message."""

    references = relationship('ArchiveSmsReference')


class ArchiveSmsReference(ArchiveReferenceMixin, BaseModel):
    """Represents an archive SMS message reference."""

    __table_args__ = (
        ForeignKeyConstraint(
            ['message_id', 'create_datetime'],
            [ArchiveSms.__table__.columns.id,  # type: ignore[attr-defined]
             ArchiveSms.__table__.columns.create_datetime]),  # type: ignore[attr-defined]
    ) + ArchiveReferenceMixin.__table_args__

    message_id = Column(Integer, nullable=False)
